'use strict';

var vue = require('vue');
var utils = require('@devops-web/utils');
var components = require('@devops-web/components');
var index = require('../useColumns/index.cjs');

const defaultPagination = () => {
    return {
        current: 1,
        pageSize: 10,
        total: 0,
        showTotal: true,
        showPageSize: true
    };
};
function useTablePage(config) {
    var _a;
    const columns = [...config.columns];
    if (config.operations) {
        const fixed = config.operationsFixed === false ? undefined : (_a = config.operationsFixed) !== null && _a !== void 0 ? _a : 'right';
        columns.push({
            title: '操作',
            dataIndex: 'operation',
            fixed,
            render(data) {
                const operations = config.operations(data.record).filter((el) => el.visible !== false);
                // @ts-ignore
                return vue.h(components.ResizeObserver, { onResize: setOperationWidth }, () => vue.h(components.OperationColumn, { operations }));
            }
        });
    }
    const loading = vue.ref(false);
    const tableData = vue.ref([]);
    const columnsHooks = index.useColumns(columns);
    vue.watch(tableData, (newVal) => {
        if (config.operations) {
            if (utils.isArray(newVal) && newVal.length > 0) {
                let flag = false;
                for (const rowData of newVal) {
                    const operations = config.operations(rowData).filter((el) => el.visible !== false);
                    if (operations && operations.length > 0) {
                        flag = true;
                        break;
                    }
                }
                columnsHooks.changeColumnVisibleByDataIndex('operation', flag);
            }
        }
    });
    function setOperationWidth(entry) {
        const { width } = entry.contentRect;
        columnsHooks.updateColumnByDataIndex('operation', 'width', width + 33);
    }
    const pagination = vue.ref(defaultPagination());
    function onPageChange(current) {
        pagination.value.current = current;
        return getTableData();
    }
    function onPageSizeChange(pageSize) {
        pagination.value.current = 1;
        pagination.value.pageSize = pageSize;
        return getTableData();
    }
    if (utils.isObject(config.pagination)) {
        pagination.value = Object.assign(Object.assign({}, defaultPagination()), config.pagination);
    }
    function getTableData() {
        loading.value = true;
        return config
            .fetch({ pageNum: pagination.value.current, pageSize: pagination.value.pageSize, columnKeys: [...columnsHooks.columnKeys.value] })
            .then((data) => {
            var _a;
            if (utils.isObject(data)) {
                tableData.value = Array.isArray(data.records) ? data.records : [];
                pagination.value.total = (_a = data.total) !== null && _a !== void 0 ? _a : 0;
            }
            else {
                console.warn('useTablePage请求返回结果数据异常!', data);
            }
        })
            .finally(() => {
            loading.value = false;
        });
    }
    /**
     * 刷新表格数据，pagination.pageNum不会变
     */
    function refreshData() {
        return getTableData();
    }
    /**
     * 加载表格数据，pagination.pageNum会设置为1
     */
    function loadTableData() {
        pagination.value.current = 1;
        return getTableData();
    }
    if (config.immediate === true) {
        loadTableData();
    }
    return {
        tableData,
        loading,
        loadTableData,
        refreshData,
        pagination,
        tableInstance: columnsHooks.tableInstance,
        originColumns: columnsHooks.originColumns,
        columns: columnsHooks.columns,
        columnKeys: columnsHooks.columnKeys,
        resetColumns: columnsHooks.resetColumns,
        changeColumnVisibleByDataIndex: columnsHooks.changeColumnVisibleByDataIndex,
        updateColumnByDataIndex: columnsHooks.updateColumnByDataIndex,
        replaceColumnByDataIndex: columnsHooks.replaceColumnByDataIndex,
        tableConfig: vue.reactive({
            loading,
            data: tableData,
            columns: columnsHooks.columns,
            ref: columnsHooks.ref,
            pagination,
            onPageChange,
            onPageSizeChange,
            onColumnResize(dataIndex, width) {
                columnsHooks.updateColumnByDataIndex(dataIndex, 'width', width);
                columnsHooks.updateColumnsWidth();
            }
        })
    };
}

exports.useTablePage = useTablePage;
